// https://discord.com/channels/237199950235041794/525510804494221312/955458285367066654
vec3 clipAABB(vec3 prevColor, vec3 minColor, vec3 maxColor) {
    vec3 pClip = 0.5 * (maxColor + minColor); // Center
    vec3 eClip = 0.5 * (maxColor - minColor); // Size

    vec3 vClip = prevColor - pClip;
    vec3 aUnit = abs(vClip / eClip);
    float denom = max(aUnit.x, max(aUnit.y, aUnit.z));

    return denom > 1.0 ? pClip + vClip / denom : prevColor;
}

vec3 neighbourhoodClipping(sampler2D currTex, vec3 prevColor, vec3 CurrentColor, ivec2 FragCoord, out vec3 maxColor) {
    vec3 a = texelFetchOffset(currTex, FragCoord, 0, ivec2(-1, -1)).rgb;
    vec3 b = texelFetchOffset(currTex, FragCoord, 0, ivec2(-1,  0)).rgb;
    vec3 c = texelFetchOffset(currTex, FragCoord, 0, ivec2(-1,  1)).rgb;

    vec3 d = texelFetchOffset(currTex, FragCoord, 0, ivec2( 0, -1)).rgb;
    vec3 e = CurrentColor;
    vec3 f = texelFetchOffset(currTex, FragCoord, 0, ivec2( 0,  1)).rgb;

    vec3 g = texelFetchOffset(currTex, FragCoord, 0, ivec2( 1, -1)).rgb;
    vec3 h = texelFetchOffset(currTex, FragCoord, 0, ivec2( 1,  0)).rgb;
    vec3 i = texelFetchOffset(currTex, FragCoord, 0, ivec2( 1,  1)).rgb;

    vec3 minColor = min(i, min(h, min(g, min(f, min(e, min(d, min(c, min(b, a))))))));
    maxColor = max(i, max(h, max(g, max(f, max(e, max(d, max(c, max(b, a))))))));

    return clipAABB(prevColor, minColor, maxColor);
}

vec3 get_closest_depth(ivec2 FragCoord, vec2 Texcoord) {
    vec3 MinDepth = vec3(FragCoord, texelFetch(depthtex1, FragCoord, 0).x);

    const ivec2[4] Offsets = ivec2[4](
        ivec2(-1, -1),
        ivec2(1, -1),
        ivec2(-1, 1),
        ivec2(1, 1)
    );
    vec4 D = textureGatherOffsets(depthtex1, Texcoord, Offsets);

    MinDepth = MinDepth.z > D.x ? vec3(FragCoord - Offsets[0], D.x) : MinDepth;
    MinDepth = MinDepth.z > D.y ? vec3(FragCoord - Offsets[1], D.y) : MinDepth;
    MinDepth = MinDepth.z > D.z ? vec3(FragCoord - Offsets[2], D.z) : MinDepth;
    MinDepth = MinDepth.z > D.w ? vec3(FragCoord - Offsets[3], D.w) : MinDepth;

    MinDepth.xy *= resolutionInv;
    return vec3(MinDepth);
}

vec3 TAA(vec3 Color, ivec2 FragCoord, vec2 Texcoord) {
    vec3 ClosestSample = get_closest_depth(FragCoord, Texcoord);
    vec2 PrevCoord = Texcoord + toPrevScreenPos(ClosestSample.xy, ClosestSample.z).xy - ClosestSample.xy;

    if (clamp(PrevCoord, 0, 1) != PrevCoord)
        return Color;

    #ifdef TAA_BICUBIC_FILTER
    vec3 PrevColor = texture_catmullrom_fast(colortex4, PrevCoord).rgb;
    #else
    vec3 PrevColor = texture(colortex4, PrevCoord).rgb;
    #endif
    if (PrevColor == vec3(0)) return Color;

    vec3 ClippingMaxColor;
    vec3 ClampedColor = neighbourhoodClipping(colortex0, PrevColor, Color, FragCoord, ClippingMaxColor);

    #if AA_MODE != 2
    float blendFactor = 0.6 + 0.2 * exp(-length((PrevCoord - Texcoord) * resolution));
    #else
    float blendFactor = 0.9;
    #endif

    // Jessie's offcenter rejection (reduce ghosting)
    vec2 pixelOffset = 1.0 - abs(2.0 * fract(PrevCoord * resolution) - 1.0);
    float OffcenterRejection = sqrt(pixelOffset.x * pixelOffset.y) * 0.15 + 0.85;
    blendFactor *= OffcenterRejection;

    #if AA_MODE == 2
    // Flicker reduction
    blendFactor = clamp(blendFactor + pow2(get_luminance((PrevColor - Color) / ClippingMaxColor) * 0.33), 0, 1);
    #endif

    Color = mix(Color, ClampedColor, blendFactor);
    return Color;
}
