#include "/lib/all_the_libs.glsl"
#include "/generic/post/cas.fsh"
in vec2 texcoord;

/* DRAWBUFFERS:0 */
layout(location = 0) out vec4 Color;

vec3 apply_vignette(vec3 Color, vec2 Pos) {
    Pos = Pos - 0.5;
    float Strength = length(Pos * VIGNETTE_OPACITY);
    Strength = pow(Strength, 4 - VIGNETTE_FALLOFF);
    Color *= 1 - min(Strength, 1);
    return Color;
}

void main() {

    #if AA_MODE == 1
    ivec2 Coords = ivec2(gl_FragCoord.xy);
    float L = texelFetchOffset(image0Sampler, Coords, 0, ivec2(0)).b;
    float R = texelFetchOffset(image0Sampler, Coords, 0, ivec2(1, 0)).a;
    float U = texelFetchOffset(image0Sampler, Coords, 0, ivec2(0)).r;
    float D = texelFetchOffset(image0Sampler, Coords, 0, ivec2(0, -1)).g;

    float Sum = L + U + R + D;
    if(Sum > 0.01) {
        bool h = max(L, R) > max(D, U);
        vec4 Offsets = h ? vec4(-L, 0, R, 0) : vec4(0, U, 0, -D);
        vec2 BlendFactor = h ? vec2(L, R) : vec2(U, D);
        BlendFactor /= dot(BlendFactor, vec2(1));

        //Offsets -= 0.5;

        vec3 FinalColor = vec3(0);
        FinalColor += textureLod(colortex0, texcoord + Offsets.xy * resolutionInv, 0).rgb * BlendFactor.x;
        FinalColor += textureLod(colortex0, texcoord + Offsets.zw * resolutionInv, 0).rgb * BlendFactor.y;

        Color.rgb = FinalColor;
        //Color.rgb += vec3(D);
    }
    else {
    #endif

        #if AA_MODE == 2
        Color.rgb = CAS(colortex0, texcoord);
        #else
        Color = textureLod(colortex0, texcoord, 0);
        #endif
    #if AA_MODE == 1
    }
    #endif

    Color.rgb = pow(Color.rgb, vec3(1 / 2.2));

    Color.rgb += (bayer8(gl_FragCoord.xy) - 0.5) / 255 * 2;

    #ifdef FILM_GRAIN
    Color.rgb = film_grain(Color.rgb, gl_FragCoord.xy);
    #endif

    Color.rgb = apply_vibrance(Color.rgb, VIBRANCE);
    Color.rgb = apply_saturation(Color.rgb, SATURATION);
    Color.rgb = apply_contrast(Color.rgb, CONTRAST);

    Color.rgb = apply_vignette(Color.rgb, texcoord);
    
    //Color.rgb += texelFetch(image0Sampler, ivec2(gl_FragCoord.xy), 0).rgb;
}
