#include "/lib/all_the_libs.glsl"

layout(local_size_x = 8, local_size_y = 8) in;

// Blend weights (SMAA)
#if AA_MODE == 1
const vec2 workGroupsRender = vec2(1.0, 1.0);
#else
const vec2 workGroupsRender = vec2(0.0, 0.0);
#endif

const float SEARCH_SIZE = 16.0;

float search_horizontal(vec2 Coord, const float Dir) {
    for(float i = 1.5; i <= SEARCH_SIZE; i += 2) {
        vec2 OffsetCoord = Coord + vec2(Dir * i, 0);

        float Sample = texture(image1Sampler, OffsetCoord * resolutionInv).g;
        if(Sample < 0.9) return i - 0.5 + Sample * 2;
    }
    return SEARCH_SIZE;
}

float search_vertical(vec2 Coord, const float Dir) {
    for(float i = 1.5; i <= SEARCH_SIZE; i += 2) {
        vec2 OffsetCoord = Coord + vec2(0, Dir * i);

        float Sample = texture(image1Sampler, OffsetCoord * resolutionInv).r;
        if(Sample < 0.9) return i - 0.5 + Sample * 2;
    }
    return SEARCH_SIZE;
}

vec2 sample_area(float d1, float d2, float e1, float e2) {
    vec2 Coord = 16 * round(4 * vec2(e1, e2)) + vec2(d1, d2);
    return texture(smaaAreaTexture, Coord / vec2(160, 560)).rg;
}

void main() {
    ivec2 GlobalPos = ivec2(gl_GlobalInvocationID.xy);
    vec2 Pos = gl_GlobalInvocationID.xy + 0.5;

    vec2 Center = imageLoad(image1, GlobalPos).rg;

    vec4 Color = vec4(0);

    // Up
    if(Center.g > 0) {
        float DistL = search_horizontal(Pos, -1);
        float DistR = search_horizontal(Pos, 1);

        float eL = texture(image1Sampler, (Pos + vec2(-DistL, 0.25)) * resolutionInv).r;
        float eR = texture(image1Sampler, (Pos + vec2(DistR, 0.25)) * resolutionInv).r;

        // DistL = sqrt(DistL);
        // DistR = sqrt(DistR);

        Color.rg = sample_area(DistL, DistR, eL, eR);
    }
    // Left
    if(Center.r > 0) {
        float DistD = search_vertical(Pos, -1);
        float DistU = search_vertical(Pos, 1);

        float eD = texture(image1Sampler, (Pos + vec2(-0.25, -DistD)) * resolutionInv).g;
        float eU = texture(image1Sampler, (Pos + vec2(-0.25, DistU)) * resolutionInv).g;

        // DistD = sqrt(DistD);
        // DistU = sqrt(DistU);

        Color.ba = sample_area(DistU, DistD, eU, eD);
    }

    imageStore(image0, GlobalPos, Color);
}
